
/*
   Basic SQLite3 tutorial using C interfaces.

   Same approach as demo found at: FB_5_7_6_Examples/Miscellaneous/SQLite_demo/SQLite_proof_of_concept
   but uses CFStrings instead of pascal strings and thus avoids those limitations.


   Change the DB path @"/Users/yourUserID/Documents/MyFirstDB"  as appropriate

   N.B. minimally tested. Error handling is marginal.

   Brian S  20130407

  Updated for FB 7.0.11, 20220215 bw
*/

include "Tlbx SQLite3.incl"

#define DB_PATH fn URLPath( fn URLByAppendingPathComponent( fn FileManagerURLForDirectory( NSDocumentDirectory, NSUserDomainMask ), @"MyFirstDB" ) )

// Function to convert CFString to C String - Ken Shmidheiser Sept. 29, 2011
local fn CopyCFStringToCString( aCFStr as CFStringRef ) as pointer

ptr     cStrBuf = NULL
CFIndex length

length = fn CFStringGetMaximumSizeForEncoding( fn CFStringGetLength( aCFStr ), _kCFStringEncodingUTF8 ) + 1
cStrBuf = fn malloc( length )
if ( cStrBuf )  then fn CFStringGetCString( aCFStr, #cStrBuf, length, _kCFStringEncodingUTF8 )
end fn = cStrBuf     // User must release with free( buffer )


local fn DoSQLExec( s as CFStringRef, dbPtr as sqlite3Ptr )

long rc
ptr  cStringPtr

print s
cStringPtr = fn CopyCFStringToCString( s )
rc = fn sqlite3_exec( dbPtr, #cStringPtr, 0, 0, 0 )
free( cStringPtr )

if ( rc ) then printf  @"  Error rc = %ld", rc
end fn



CFStringRef dbName = @"Opening MyFirstDB"
sqlite3Ptr  myDbHandle
long        rc
ptr         cStringPtr

window 1, @"SQLite Demo", (0,0,500,500)

// announce
printf @"SQLite version: %d",fn sqlite3_libversion_number

// open DB
print dbName
cStringPtr = fn CopyCFStringToCString( DB_PATH )
rc = fn sqlite3_open( #cStringPtr, @myDbHandle )
if ( rc ) then printf @"  Error rc = %ld", rc
free( cStringPtr )

// populate DB
fn DoSQLExec( @"DROP TABLE IF EXISTS memos;", myDbHandle )
fn DoSQLExec( @"CREATE TABLE memos( task TEXT, priority INTEGER );", myDbHandle )
fn DoSQLExec( @"INSERT INTO memos VALUES( 'finish this demo', 10 );", myDbHandle )
fn DoSQLExec( @"INSERT INTO memos VALUES( 'drink beer', 100 );", myDbHandle )

// query how many rows changed
printf @"Rows changed: %d", fn sqlite3_total_changes( myDBHandle )

// close DB
fn sqlite3_close( myDbHandle )

HandleEvents

